/* ------------------------------------------------------------------------------------------------ */
/* FILE :        sio8051.c																			*/
/*																									*/
/* CONTENTS :																						*/
/*               Abassi BSP: UART driver for 8051													*/
/*																									*/
/*																									*/
/* Copyright (c) 2011, Code-Time Technologies Inc. All rights reserved.								*/
/*																									*/
/* Code-Time Technologies retains all right, title, and interest in and to this work				*/
/*																									*/
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS							*/
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF										*/
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL							*/
/* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR								*/
/* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,							*/
/* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR							*/
/* OTHER DEALINGS IN THE SOFTWARE.																	*/
/*																									*/
/*																									*/
/*	$Revision: 1.10 $																				*/
/*	$Date: 2014/04/23 16:57:23 $																	*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

#include "sio8051.h"

static void SerialPut(char c) reentrant;			/* Put a character to the serial port (pre ISR)	*/

volatile __bit G_SerialWait4room = 1;				/* Default: if out buffer full, wait for room	*/
static   __bit g_TxLatent;							/* Flag indicating out is currently going on	*/
volatile __bit G_Sio8051UseCTS   = 0;				/* If user ask us to use the handshake			*/
volatile __bit G_Sio8051UseISR;						/* If user ask us to use the ISR & buffers		*/

#ifndef OS_SIO_BUF_SIZE
  #define OS_SIO_BUF_SIZE 16						/* Circular buffer size (must be 2^N)			*/
#endif
#if (OS_SIO_BUF_SIZE)&(OS_SIO_BUF_SIZE-1)			/* Verify it's a power of 2						*/
	#error "OS_SIO_BUF_SIZE must be a power of 2"
#endif
#define OS_SIO_BUF_MASK (OS_SIO_BUF_SIZE-1)			/* Wrap around mask for the circular buffer		*/

static char g_TxBuf[OS_SIO_BUF_SIZE];				/* Transmission circular buffer					*/
static char g_RxBuf[OS_SIO_BUF_SIZE];				/* Reception circular buffer					*/

static volatile __near int g_RxIdxIn;				/* Circular buffer read index for reception		*/
static volatile __near int g_RxIdxOut;				/* Circular buffer write index for recpetion	*/
static volatile __near int g_TxIdxIn;				/* Circular buffer read index for transmission	*/
static volatile __near int g_TxIdxOut;				/* Circular buffer write index for transmission	*/

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    Sio8051Init																			*/
/*																									*/
/* Sio8051Init - Initialize the driver for the UART													*/
/*																									*/
/* SYNOPSIS:																						*/
/*		void Sio8051Init(int BaudRate, int UseISR, int TimNmb, int Prio)							*/
/*																									*/
/* ARGUMENTS:																						*/
/*		BaudRate : Baudrate to set the UARt to														*/
/*		           If is 0, the value sp[ecifed by OS_SERIAL_BAUD is used							*/
/*		UseISR   : Boolean indicating if ISRs are used or not										*/
/*		           == 0: polling is used															*/
/*		           != 0: ISRs are used																*/
/*		TimNmb   : Timer number to use: can only have the values 1 or 2								*/
/*		Prio     : when ISRs are used, priority of the interrupt									*/
/*		           == 0: low priority																*/
/*		           != 0: high priority																*/
/*																									*/
/* RETURN VALUE:																					*/
/*		void																						*/
/*																									*/
/* DESCRIPTION:																						*/
/* ------------------------------------------------------------------------------------------------ */

void Sio8051Init(int BaudRate, int UseISR, int TimNmb, int Prio)

{
unsigned int Reload;								/* Counter-timer reload value					*/
         int PastEA;


	PastEA = EA;									/* Memo in case we are not using ISRs			*/
	EA     = 0;										/* Disable Interrupts in case is a re-init		*/

  #ifdef OS_SERIAL_BAUD
	if (BaudRate == 0) {
		BaudRate = OS_SERIAL_BAUD;
	}
  #endif

	if (TimNmb == 1) {								/* Timer 1 requested to be used					*/
		Reload = 0xFF
		       & (256 - (int)(((long)(OS_CPU_FREQ)) / (192L * (long)BaudRate)));
		TMOD   = 0x20								/* Timer 0 untouched, Timer 1 in mode 2			*/
		       | (0x0F & TMOD);
		TH1    = Reload;							/* Reload value (periodic)						*/
		TL1    = Reload;							/* Start the timer in known state				*/
		TR1    = 1;									/* Start the timer: run bit set to 1			*/
		PCON  |= 0x80;								/* Set the SMOD bit in PCON for double baudrate	*/
	}
	else {											/* Timer 2 requested to be used					*/
		Reload = -((int)((long)(OS_CPU_FREQ) / (32L * (long)BaudRate)));
		RCAP2H = (Reload>>8) & 0xFF;				/* Set-up timer 2 reload values					*/
		RCAP2L =  Reload     & 0xFF;
		T2CON  = 0x34;								/* Set-up Timer 2 as baudrate generator			*/
	}
	SCON = 0x52;									/* SIO mode 1, enable RX, raise TX interrupt	*/


	if (UseISR != 0) {								/* If we use the ISRs, enable them				*/
		PS       = (Prio == 0)						/* Set the serial I/O to request prio			*/
		         ? 0
		         : 1;
		ES       = 1;								/* Enable the serial port interrupts			*/
		PastEA   = 1;								/* Will be done at the last minute				*/
	}
	G_Sio8051UseISR = UseISR;

	g_TxLatent  = 1;
	g_RxIdxIn   = 0;								/* Init the r/w indexes							*/
	g_RxIdxOut  = 0;
	g_TxIdxIn   = 0;
	g_TxIdxOut  = 0;
	if (G_Sio8051UseCTS != 0) {
		CTS_PIN = 0;								/* Ready to accept data							*/
	}

	REN = 1;										/* Enable the reception							*/
	EA  = PastEA;									/* Restore/force interrupt enabling				*/

	return;
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    GetKey																				*/
/*																									*/
/* GetKey - report if a new character is available on the serial port								*/
/*																									*/
/* SYNOPSIS:																						*/
/*		void Sio8051Init(int BaudRate, int UseISR, int Timer)										*/
/*																									*/
/* ARGUMENTS:																						*/
/*		void																						*/
/*																									*/
/* RETURN VALUE:																					*/
/*		int : == 0 means no new character available													*/
/*		      != 0 new character read from the SIO													*/
/*																									*/
/* DESCRIPTION:																						*/
/* When the serial port is operating in polling mode, this function returns the value of the		*/
/* RI bit.																							*/
/* When the serial port is operating in interrupt mode, the contents of the reception circular		*/
/* buffer is checked to see if one or more new characters have been received						*/
/* ------------------------------------------------------------------------------------------------ */

int GetKey(void)
reentrant
{
int  RetVal;
char Tmp;


	RetVal = 0;										/* Assume no character available				*/

	if (G_Sio8051UseISR == 0) {						/* Polling method								*/
		if (RI != 0) {								/* RI set when character available				*/
			RetVal = SBUF;							/* Grab the character							*/
			RI = 0;									/* Clear this, we have grabbed the character	*/
		}
	}
	else {											/* Serial is in interrupt mode					*/
		if (g_RxIdxOut != g_RxIdxIn) {				/* Comparing indexes is enough: when same empty	*/
			RetVal     = g_RxBuf[g_RxIdxOut];		/* Grab the oldest character					*/
			g_RxIdxOut = (g_RxIdxOut + 1)			/* Update the extraction index					*/
			           & OS_SIO_BUF_MASK;
			if (G_Sio8051UseCTS != 0) {
				Tmp = (g_RxIdxIn - g_RxIdxOut)		/* Check how much room left in the buffer		*/
				    & OS_SIO_BUF_MASK;				/* Tmp now holds # of entries used				*/
				if (Tmp < (OS_SIO_BUF_MASK-3)) {	/* Buffer has more than 3 free entries left		*/
					CTS_PIN = 0;					/* The other end can keep sending				*/
				}
			}
		}											/* When out index != in index					*/
	}

	return(RetVal);
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    getchar																				*/
/*																									*/
/* getchar - standard stdio library function (overloads the library getchar())						*/
/*																									*/
/* SYNOPSIS:																						*/
/*		char getchar(void)																			*/
/*																									*/
/* ARGUMENTS:																						*/
/*		void																						*/
/*																									*/
/* RETURN VALUE:																					*/
/*		char : newly received character on the serial port											*/
/*		      If was standard library should be int and not char but Keil has it as char			*/
/*																									*/
/* DESCRIPTION:																						*/
/* When the serial port is operating in polling mode, this function waits until the SFR bit RI		*/
/* is set.  It then reads the newlyu received character												*/
/* When the serial port is operating in interrupt mode, the circular buffer is checked				*/
/* If the buffer is empty, it waits for it to have a new character deposited.						*/
/* The oldest character ios extracted from the buffer and retuned to the caller						*/
/* ------------------------------------------------------------------------------------------------ */

char getchar(void)
{

char c;
char Tmp;


	if (G_Sio8051UseISR == 0) {						/* Not using the interrupts, manual work		*/
		for ( ; RI==0 ; );							/* Wait for the arrival of a character			*/
		c  = SBUF;									/* Grab the charater							*/
		RI = 0;										/* Clear this, we have grabbed the character	*/
	}
	else {											/* Interrupt transfers							*/
		for ( ; g_RxIdxIn == g_RxIdxOut ; );		/* Wait for a character to land in the buffer	*/
		c          = g_RxBuf[g_RxIdxOut];			/* Grab the oldest character					*/
		g_RxIdxOut = (g_RxIdxOut + 1)				/* Update the extraction index					*/
		           & OS_SIO_BUF_MASK;
		if (G_Sio8051UseCTS != 0) {
			Tmp = (g_RxIdxIn - g_RxIdxOut)			/* Check how much room left in the buffer		*/
			    & OS_SIO_BUF_MASK;					/* Tmp now holds # of entries used				*/
			if (Tmp < (OS_SIO_BUF_MASK-3)) {		/* Buffer has more than 3 free entries left		*/
				CTS_PIN = 0;						/* The other end can keep sending				*/
			}
		}
	}

	if (c == '\r') {								/* If CR, replace it with a LF					*/
		c = '\n';
	}

	return(c);
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    putchar																				*/
/*																									*/
/* putchar - standard stdio library function (overloads the library putchar())						*/
/*																									*/
/* SYNOPSIS:																						*/
/*		char putchar(int c)																			*/
/*																									*/
/* ARGUMENTS:																						*/
/*		c    : character to send on the serial port													*/
/*																									*/
/* RETURN VALUE:																					*/
/*		char : return c																				*/
/*																									*/
/* DESCRIPTION:																						*/
/* When the character is a new line, a cariage retuyrn is first sent out then the new line			*/
/* This function uses the local function SerilaPut()												*/
/* ------------------------------------------------------------------------------------------------ */

char putchar(char c)
{

	if (c == '\n') {								/* If end of line, send CR before				*/
		SerialPut('\r');							/* sending LF									*/
	}

	SerialPut((char)c);

	return(c);
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    SerialPut																			*/
/*																									*/
/* SerialPut - send a character on the serrial port													*/
/*																									*/
/* SYNOPSIS:																						*/
/*		 void SerialPut(char c)																		*/
/*																									*/
/* ARGUMENTS:																						*/
/*		c    : character to send on the serial port													*/
/*																									*/
/* RETURN VALUE:																					*/
/*		void																						*/
/*																									*/
/* DESCRIPTION:																						*/
/* ------------------------------------------------------------------------------------------------ */

static void SerialPut(char c)
reentrant
{
         int PastEA;
volatile int Tmp;

	if (G_Sio8051UseISR == 0) {						/* Not using the interrupts, manual work		*/
		Tmp = 0xFFFF;								/* Keil simulator loses the TI bit sometimes	*/
		for ( ; (TI==0) && (Tmp!=0) ; Tmp--);		/* Wait for room in the serial buffer			*/
		SBUF = c;									/* and here it is								*/
		TI   = 0;									/* Clear it, we have filled the buffer			*/
	}
	else {											/* Interrupt transfers							*/
		Tmp = (g_TxIdxIn + 1)						/* Next write index								*/
		    & OS_SIO_BUF_MASK;
		if (Tmp == g_TxIdxOut) {					/* Buffer is full, and requested to not wait	*/
			if   (G_SerialWait4room == 0) {			/* so exit right now							*/
				return;
			}
			else {									/* Request to wait for room in the buffer		*/
				for ( ; Tmp == g_TxIdxOut ; );
			}
		}

		g_TxBuf[g_TxIdxIn] = c;						/* Insert the character in the queue			*/

		PastEA = EA;
		EA     = 0;

		g_TxIdxIn = Tmp;							/* Update the write index index					*/
		if (g_TxLatent != 0) {						/* Last ISR saw nothing in the serial buffer	*/
			TI = 1;									/* Set TI to trigger a SIO interrupt			*/
			g_TxLatent = 0;							/* There is something in the serial buffer now	*/
		}
		EA = PastEA;								/* Re-enable the interrupts						*/
	}

	return;
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    HWIsio8051																			*/
/*																									*/
/* HWIsio8051 - interrupt handler for the serial port												*/
/*																									*/
/* SYNOPSIS:																						*/
/*		 void  HWIsio8051(void)																		*/
/*																									*/
/* ARGUMENTS:																						*/
/*		void																						*/
/*																									*/
/* RETURN VALUE:																					*/
/*		void																						*/
/*																									*/
/* DESCRIPTION:																						*/
/* ------------------------------------------------------------------------------------------------ */

void HWIsio8051(void)
reentrant
{

char c;												/* Character in transit							*/
int  Tmp;											/* General purpose								*/

	do {
		if (RI == 1) {								/* Is there a received character?				*/
			RI = 0;									/* Clear it, we are grabbing it					*/
			Tmp = (g_RxIdxIn - g_RxIdxOut)			/* Check how much room left in the buffer		*/
			    & OS_SIO_BUF_MASK;					/* Tmp hold ho wmany entries are used			*/
			if ((Tmp < (OS_SIO_BUF_MASK-3))			/* Buffer only has 3 entries left in.			*/
			&& 	(G_Sio8051UseCTS != 0)) {
				CTS_PIN = 1;						/* Tell the other end to stop ending			*/
			}
			c  = SBUF;								/* Grab the charater							*/
			Tmp = (g_RxIdxIn + 1)
			    & OS_SIO_BUF_MASK;
			if (Tmp != g_RxIdxOut) {				/* Memo only if there is room in the buffer		*/
				g_RxBuf[g_RxIdxIn] = c;				/* Put it in the buffer							*/
				g_RxIdxIn          = (g_RxIdxIn + 1)	/* Move to the next index					*/
				                   & OS_SIO_BUF_MASK;	/* And buffer wrap around					*/
			}
		}

		if (TI == 1) {								/* Has a character just been transmitted?		*/
			TI = 0;									/* And clear the interrupt flag					*/
			if (g_TxIdxOut != g_TxIdxIn) {			/* Check if there is stuff in the buffer		*/
				SBUF = g_TxBuf[g_TxIdxOut];			/* Put the character in the serial buffer		*/
				g_TxIdxOut = (g_TxIdxOut + 1)
				           & OS_SIO_BUF_MASK;
				g_TxLatent = 0;						/* There is something in the buffer				*/
			}
			else {									/* Nothing in queue... neither in sbuf			*/ 
				g_TxLatent = 1;
			}
		}
	} while ((TI != 0) || (RI != 0));

	return;
}

/* EOF */
