/* ------------------------------------------------------------------------------------------------ */
/* FILE :        tim8051.c																			*/
/*																									*/
/* CONTENTS :																						*/
/*               Abassi BSP: Timer driver for 8051													*/
/*																									*/
/*																									*/
/* Copyright (c) 2011, Code-Time Technologies Inc. All rights reserved.								*/
/*																									*/
/* Code-Time Technologies retains all right, title, and interest in and to this work				*/
/*																									*/
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS							*/
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF										*/
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL							*/
/* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR								*/
/* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,							*/
/* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR							*/
/* OTHER DEALINGS IN THE SOFTWARE.																	*/
/*																									*/
/*																									*/
/*	$Revision: 1.8 $																				*/
/*	$Date: 2014/04/21 18:51:25 $																	*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

#include "tim8051.h"

static int    g_Reload0;							/* Reload value for timer 0						*/
static int    g_Reload1;							/* Reload value for timer 1						*/
static void  (* g_CallB0)(void);					/* Call back function at every interrupts		*/
static void  (* g_CallB1)(void);					/* Call back function at every interrupts		*/
static void  (* g_CallB2)(void);					/* Call back function at every interrupts		*/
static int    g_T0shot1;							/* When the timer is requested to be a 1 shot	*/
static int    g_T1shot1;							/* When the timer is requested to be a 1 shot	*/
static int    g_T2shot1;							/* When the timer is requested to be a 1 shot	*/

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    Tim8051Init																			*/
/*																									*/
/* Tim8051Init - Initialize a timer driver															*/
/*																									*/
/* SYNOPSIS:																						*/
/*		void Tim8051Init(int TimNmb, int Period, int Prio, void (* CallB)(void))					*/
/*																									*/
/* ARGUMENTS:																						*/
/*		TimNmb   : Timer number to set-up.  Only valid values are 0,1 and 2							*/
/*		Period   : Value to set for the period of the timer.										*/
/*		           Is in number of processor cycles													*/
/*		           Use  TIM8051_RLD() to convert micro seconds into processor cycles				*/
/*		Prio     : when ISRs are used, priority of the interrupt									*/
/*                 Value must be between 0 and 3													*/
/*		Timer    : Timer number to use: can only have the values 1 or 2								*/
/*		CallB    : When non-NULL, call back function												*/
/*		OneShot  : Boolean: <>0 is 1 shot timer, ==0 is peridic										*/
/*																									*/
/* RETURN VALUE:																					*/
/*		void																						*/
/*																									*/
/* DESCRIPTION:																						*/
/* ------------------------------------------------------------------------------------------------ */

void Tim8051Init(int TimNmb, int Period, int Prio, void (* CallB)(void), int OneShot)

{
__bit PastEA;


	Period = -Period;								/* The timers are up counters					*/
	PastEA = EA;									/* Memo current interrupt enable / disable		*/
	EA     = 0;										/* Disable Interrupts in case is a re-init		*/

	if (TimNmb == 0) {								/* Programming Timer 0 for periodic interrupts	*/
		g_T0shot1 = OneShot;
		g_CallB0  = CallB;							/* Memo the call back function pointer			*/
		TR0       = 0;								/* Stop the timer in case was running			*/
		TH0       = (Period>>8)&0xFF;
		TL0       =  Period    &0xFF;
		g_Reload0 =  Period;
		TMOD      = 0x01 | (0xF0 & TMOD);			/* Set to mode 1, internal clock & not gated	*/
		PT0       = (Prio == 0)						/* Set the timer to the requested priority		*/
		          ? 0
			      : 1;
		ET0       = 1;								/* Enable timer 0 overflow interrupts			*/
		TR0       = 1;								/* Start the timer								*/
	}
	else if (TimNmb == 1) {							/* Programming Timer 1 for periodic interrupts	*/
		g_CallB1  = CallB;							/* Memo the call back function pointer			*/
		g_T1shot1 = OneShot;
		TR1       = 0;								/* Stop the timer in case was running			*/
		TH1       = (Period>>8)&0xFF;
		TL1       =  Period    &0xFF;
		g_Reload1 =  Period;
		TMOD      = 0x10 | (0x0F & TMOD);			/* Set to mode 1, internal clock & not gated	*/
		PT1       = (Prio == 0)						/* Set the timer to the requested priority		*/
		          ? 0
			      : 1;
		ET1       = 1;								/* Enable timer 1 overflow interrupts			*/
		TR1       = 1;								/* Start the timer								*/
	}
	else {											/* Programming Timer 2 for periodic interrupts	*/
		g_CallB2  = CallB;							/* Memo the call back function pointer			*/
		g_T2shot1 = OneShot;
		RCAP2H    = (Period>>8)&0xFF;				/* Set-up the 16 bit reload register			*/
		RCAP2L    =  Period    &0xFF;
		PT2       = (Prio == 0)						/* Set the timer to the requested priority		*/
		          ? 0
			      : 1;
		ET2       = 1;								/* Enable timer 2 overflow interrupts			*/
		T2CON     = 0x04;							/* Internal clock, auto-reload & start timer	*/
	}

	EA = PastEA;									/* Put ISR back to what they were				*/

	return;
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    HWItim8051_0																		*/
/*																									*/
/* HWItim8051_0 - interrupt handler for timer #0													*/
/*																									*/
/* SYNOPSIS:																						*/
/*		 void  HWItim8051_0(void)																	*/
/*																									*/
/* ARGUMENTS:																						*/
/*		void																						*/
/*																									*/
/* RETURN VALUE:																					*/
/*		void																						*/
/*																									*/
/* DESCRIPTION:																						*/
/* ------------------------------------------------------------------------------------------------ */

void HWItim8051_0(void)
reentrant
{
int PastEA;
int TimNow;

	PastEA = EA;									/* We are called from ISR but EA can be == 0	*/
	EA     = 0;										/* Can't be interrupted: will screw up the time	*/
	TR0    = 0;										/* Stop the timer								*/
	if (g_T0shot1 == 0) {							/* This is a periodic timer						*/
		TimNow = (((int)TH0)<< 8) 					/* Read how much time has elapsed since the 	*/
		       |   TL0;								/* timer rolled over							*/
		TimNow = g_Reload0 + TimNow + 93;			/* Add the cycle count we are spending in here	*/
		TH0    = (TimNow>>8)&0xFF;					/* between disable the timer & enabling it		*/
		TL0    =  TimNow&0xFF;
		TR0    = 1;									/* Re-enable the timer							*/
	}
	if (PastEA != 0) {
		EA = 1;										/* Put interrupt enable as upon entry			*/
	}
	if (g_CallB0 != NULL) {							/* If a call back function has been provided,	*/
		g_CallB0();									/* call it										*/
	}
	return;
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    HWItim8051_1																		*/
/*																									*/
/* HWItim8051_1 - interrupt handler for timer #1													*/
/*																									*/
/* SYNOPSIS:																						*/
/*		 void  HWItim8051_1(void)																	*/
/*																									*/
/* ARGUMENTS:																						*/
/*		void																						*/
/*																									*/
/* RETURN VALUE:																					*/
/*		void																						*/
/*																									*/
/* DESCRIPTION:																						*/
/* ------------------------------------------------------------------------------------------------ */

void HWItim8051_1(void)
reentrant
{
int PastEA;
int TimNow;

	PastEA = EA;									/* We are called from ISR but EA can be == 0	*/
	EA     = 0;										/* Can't be interrupted: will screw up the time	*/
	TR1    = 0;										/* Stop the timer								*/
	if (g_T1shot1 == 0) {							/* This is a periodic timer						*/
		TimNow = (((int)TH1)<< 8) 					/* Read how much time has elapsed since the 	*/
		       |   TL1;								/* timer rolled over							*/
		TimNow = g_Reload1 + TimNow + 93;			/* Add the cycle count we are spending in here	*/
		TH1    = (TimNow>>8)&0xFF;					/* between disable the timer & enabling it		*/
		TL1    =  TimNow&0xFF;
		TR1    = 1;									/* Re-enable the timer							*/
	}

	if (PastEA != 0) {
		EA = 1;										/* Put interrupt enable as upon entry			*/
	}
	if (g_CallB1 != NULL) {							/* If a call back function has been provided,	*/
		g_CallB1();									/* call it										*/
	}
	return;
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    HWItim8051_2																		*/
/*																									*/
/* HWItim8051_2 - interrupt handler for timer #2													*/
/*																									*/
/* SYNOPSIS:																						*/
/*		 void  HWItim8051_2(void)																	*/
/*																									*/
/* ARGUMENTS:																						*/
/*		void																						*/
/*																									*/
/* RETURN VALUE:																					*/
/*		void																						*/
/*																									*/
/* DESCRIPTION:																						*/
/* ------------------------------------------------------------------------------------------------ */

void HWItim8051_2(void)
reentrant
{

	TF2  = 0;										/* For Timer 2 ISRs, always need to clear both	*/
	EXF2 = 0;										/* bits											*/
	if (g_CallB2 != NULL) {							/* If a call back function has been provided,	*/
		g_CallB2();									/* call it										*/
	}
	return;
}


/* EOF */
																							 